/*
 * Part of the GaBIEn project.
 * (Graphics and Basic Input Engine)
 *
 *        DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *                    Version 2, December 2004 
 *
 * Modified by gamemanj <asdd2808@gmail.com> to remove expletives
 * Original copyright (C) 2004 Sam Hocevar <sam@hocevar.net> 
 *
 * Everyone is permitted to copy and distribute verbatim or modified 
 * copies of this license document, and changing it is allowed as long 
 * as the name is changed. 
 *
 *            DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION 
 *
 *  0. You just DO WHATEVER YOU WANT TO.
 */
package gabien;

import java.io.FileInputStream;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.sound.sampled.LineUnavailableException;

/**
 * Graphics and Basic Intput-english
 * Subsystems should be initialized in this order:
 * graphics,sound
 * @author matt
 */
public final class GaBIEnImpl implements IGaBIEn {
    //In case you get confused.
    private long startup=System.currentTimeMillis();
    public double getTime()
    {
        return (System.currentTimeMillis()-startup)/1000.0;
    }
    private double lastdt=getTime();
    public double timeDelta(boolean reset)
    {
        double dt=getTime()-lastdt;
        if (reset)
        lastdt=getTime();
        return dt;
    }
    public InputStream getResource(String resource)
    {
        return ClassLoader.getSystemClassLoader().getResourceAsStream(resource);
    }
    public InputStream getFile(String FDialog) {
    	try {
    		return new FileInputStream(FDialog);
    	} catch (Exception e)
    	{
    		return null;
    	}
    }
    public IGrInDriver makeGrIn(String name,int w,int h,WindowSpecs ws)
    {
        return new gabien.GrInDriver(name,ws.scale,w,h);
    }
    public boolean singleWindowApp()
    {
    	return false;
    }
    private ISoundDriver sound=null;
    public void ensureQuit() {
        System.exit(0);
    }

    public ISoundDriver getSound() {
        if (sound==null)
        {
            try {
                sound=new gabien.SoundDriver();
            } catch (LineUnavailableException ex) {
                Logger.getLogger(GaBIEnImpl.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        return sound;
    }
	@Override
	public WindowSpecs defaultWindowSpecs(String name, int w, int h) {
		WindowSpecs ws=new WindowSpecs();
		ws.scale=((w>400)||(h>300))?1:2;
		return ws;
	}

}
