/*
 * Part of the JavaChan project.
 * (Java ikaChan)
 *
 *        DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *                    Version 2, December 2004 
 *
 * Modified by gamemanj <asdd2808@gmail.com> to remove expletives
 * Original copyright (C) 2004 Sam Hocevar <sam@hocevar.net> 
 *
 * Everyone is permitted to copy and distribute verbatim or modified 
 * copies of this license document, and changing it is allowed as long 
 * as the name is changed. 
 *
 *            DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION 
 *
 *  0. You just DO WHATEVER YOU WANT TO.
 */
package javachan;

import gabien.ui.Rect;

import java.util.LinkedList;
import java.util.List;

import javachan.PhysicsEngine;
import javachan.blocks.BasicBlockType;
import javachan.bosses.BasicBoss;

/**
 * "UNLESS someone like you cares a whole awful lot,nothing will ever change."
 * Makes about as much sense to people who like NXEngine for Ikachan, as it does
 * to anything else.
 * 
 * @author matt
 */
public class PhysicsEngine {

    public PhysicsEngine() {
    }

    // Note that it's the PC collisions that create events.This is important.
    // Furthermore,the whole Capacitor thing is one massive special case-
    // -1:Not needed,0:Left,1:Right,2:Up,3:Down

    private int pushOutObjectFromObject(IPhysicsObject po, Rect r) {
        if (r.intersects(po.getRectangle())) {
            double oX = po.getX();
            double oY = po.getY();
            int p0 = pushOutObjectPass(po, r, -1, 0);
            po.setX(oX);
            po.setY(oY);
            int p1 = pushOutObjectPass(po, r, 1, 0);
            po.setX(oX);
            po.setY(oY);
            int p2 = pushOutObjectPass(po, r, 0, -1);
            po.setX(oX);
            po.setY(oY);
            int p3 = pushOutObjectPass(po, r, 0, 1);
            po.setX(oX);
            po.setY(oY);
            // Okay,now find which is better.
            int bestind = 2;
            int besttime = Integer.MAX_VALUE;
            int[] arr = new int[] { p0, p1, p2, p3 };
            for (int s = 0; s < 4; s++) {
                if (arr[s] < besttime) {
                    besttime = arr[s];
                    bestind = s;
                }
            }
            if (bestind == 0) {
                pushOutObjectPass(po, r, -1, 0);
            }
            if (bestind == 1) {
                pushOutObjectPass(po, r, 1, 0);
            }
            if (bestind == 2) {
                pushOutObjectPass(po, r, 0, -1);
            }
            if (bestind == 3) {
                pushOutObjectPass(po, r, 0, 1);
            }
            return bestind;
        }
        return -1;
    }

    private int pushOutObjectFromWorld(IPhysicsObject po, GameStateMap r, GameState gs) {
        double oX = po.getX();
        double oY = po.getY();
        int bestind = 0;
        int besttime = pushOutWorldPass(po, r, -1, 0,Integer.MAX_VALUE);
        po.setX(oX);
        po.setY(oY);
        int tRecord = pushOutWorldPass(po, r, 1, 0, besttime);
        if (tRecord<besttime)
        {
            besttime=tRecord;
            bestind=1;
        }
        po.setX(oX);
        po.setY(oY);
        tRecord = pushOutWorldPass(po, r, 0, -1, besttime);
        if (tRecord<besttime)
        {
            besttime=tRecord;
            bestind=2;
        }
        po.setX(oX);
        po.setY(oY);
        tRecord = pushOutWorldPass(po, r, 0, 1, besttime);
        if (tRecord<besttime)
        {
            besttime=tRecord;
            bestind=3;
        }
        po.setX(oX);
        po.setY(oY);
        if (bestind == 0) {
            pushOutWorldPass(po, r, -1, 0, besttime+1);
        }
        if (bestind == 1) {
            pushOutWorldPass(po, r, 1, 0, besttime+1);
        }
        if (bestind == 2) {
            pushOutWorldPass(po, r, 0, -1, besttime+1);
        }
        if (bestind == 3) {
            pushOutWorldPass(po, r, 0, 1, besttime+1);
        }
        if (besttime > 16) {
            System.out.println("REALLY BAD! Potentially warping object! ("+bestind+") ");
        }
        return bestind;
    }

    public int utilClamp(int n, int min, int max) {
        if (n < min) {
            n = min;
        }
        if (n > max) {
            n = max;
        }
        return n;
    }

    public int runPushOutOfMapPass(IPhysicsObject op, GameState gs, boolean soft) {
        GameStateMap map = gs.map;
        int cx = (int) op.getX();
        int cl = cx - 1;
        int cr = cx + 1;
        cl = utilClamp(cl, 0, map.width - 1);
        cr = utilClamp(cr, 0, map.width - 1);
        int cy = (int) op.getY();
        int cu = cy - 1;
        int cd = cy + 1;
        cu = utilClamp(cu, 0, map.height - 1);
        cd = utilClamp(cd, 0, map.height - 1);
        List<IntegerPoint> blocks = new LinkedList<IntegerPoint>();
        for (int px = cl; px <= cr; px++) {
            for (int py = cu; py <= cd; py++) {
                int md = map.data[px + (py * map.width)];
                BasicBlockType bbt = map.blockTypes[md / 16];
                if (bbt != null) {
                    Rect br = bbt.getCollision(px, py, md % 16);
                    if (br.intersects(op.getRectangle())) {
                        if (bbt.collisionStops() || soft) {
                            IntegerPoint ip = new IntegerPoint();
                            ip.X = px;
                            ip.Y = py;
                            blocks.add(ip);
                        }
                    }
                }
            }
        }
        if (blocks.size() > 0) {
            if (soft) {
                for (IntegerPoint ip : blocks) {
                    int md = map.data[ip.X + (ip.Y * map.width)];
                    BasicBlockType bbt = map.blockTypes[md / 16];
                    bbt.handleSoftCollision(ip.X, ip.Y, gs, op);
                }
            } else {
                int d = pushOutObjectFromWorld(op, map,gs);
                for (IntegerPoint ip : blocks) {
                    int md = map.data[ip.X + (ip.Y * map.width)];
                    BasicBlockType bbt = map.blockTypes[md / 16];
                    bbt.handleHardCollision(ip.X, ip.Y, gs, op, d);
                }
                return d;
            }
        }
        return -1;
    }

    public int runPushOutOfObjectPass(IPhysicsObject op, GameState gs,
            double Time, IEventCallback iec) {
        GameStateNPChar map = gs.npcs;
        for (int px = 0; px < map.npcTable.length; px++) {
            if (map.npcTable[px] == op) {
                continue;
            }
            if (!map.npcTable[px].exists) {
                continue;
            }
            // collisiontype:0:NULL 1:BLOCK 2:ENEMY 3:EVENT
            int ctype = map.npcTable[px].collisionType;
            boolean Colliding = map.npcTable[px].getRectangle().intersects(
                    op.getRectangle());
            if (Colliding) {
                if ((ctype == 0) || (ctype == 3)) {
                    if (op == gs) {
                        map.npcTable[px].handleIkachanCollision(gs, iec, Time);
                    }
                }
                if ((ctype == 2) || (ctype == 3)) {
                    int r = pushOutObjectFromObject(op,
                            map.npcTable[px].getRectangle());
                    if (gs == op) {
                        if (ctype == 2) {
                            map.npcTypes[map.npcTable[px].entityType]
                                    .ikachanTouchedEnemy(map.npcTable[px], gs, r,
                                            iec);
                        }
                    }
                    if ((r == 3) || (r == 2)) {
                        op.setVY(0);
                    }
                    if ((r == 1) || (r == 0)) {
                        op.setVX(0);
                    }
                    return r;
                }
            }
        }
        return -1;
    }

    private boolean[] collisionPass(IPhysicsObject op, GameState gs,
            double Time, IEventCallback iec, boolean firstpass) {
        boolean[] ret = new boolean[4];
        boolean IsIkachan = (op == gs);

        int r = runPushOutOfMapPass(op, gs, false);
        if (r != -1) {
            ret[r] = true;
        }
        if (IsIkachan) {
            r = runPushOutOfObjectPass(op, gs, Time, iec);
            if (r != -1) {
                ret[r] = true;
            }
        }
        if (firstpass) {
            runPushOutOfMapPass(op, gs, true);
            if (IsIkachan) {
                for (BasicBoss boss : gs.bosses) {
                    if (boss.isAlive(gs)) {
                        int hitironhead = pushOutObjectFromObject(op,
                                boss.getRectangle());
                        if (hitironhead != -1) {
                            ret[hitironhead] = true;
                            boss.onHit(gs, hitironhead, iec, Time);
                        }
                    }
                }
            }
        }
        return ret;
    }

    private int pushOutObjectPass(IPhysicsObject po, Rect r, int i, int i0) {
        int time = 0;
        while (po.getRectangle().intersects(r)) {
            po.setX(((int)((po.getX()*16) + i))/16.0d);
            po.setY(((int)((po.getY()*16) + i0))/16.0d);
            time++;
        }
        return time;
    }

    private int pushOutWorldPass(IPhysicsObject po, GameStateMap map, int i,
            int i0, int quitAt) {
        int time = 0;
        while (time<quitAt) {
            int cx = (int) Math.floor(po.getX());
            int cl = cx - 1;
            int cr = cx + 1;
            cl = utilClamp(cl, 0, map.width - 1);
            cr = utilClamp(cr, 0, map.width - 1);
            int cy = (int) Math.floor(po.getY());
            int cu = cy - 1;
            int cd = cy + 1;
            cu = utilClamp(cu, 0, map.height - 1);
            cd = utilClamp(cd, 0, map.height - 1);
            boolean intersected = false;
            for (int px = cl; px <= cr; px++) {
                for (int py = cu; py <= cd; py++) {
                    int md = map.data[px + (py * map.width)];
                    BasicBlockType bbt = map.blockTypes[md / 16];
                    if (bbt != null) {
                        Rect br = bbt.getCollision(px, py, md % 16);
                        if (br.intersects(po.getRectangle())) {
                            if (bbt.collisionStops()) {
                                intersected = true;
                                break;
                            }
                        }
                    }
                }
                if (intersected)
                    break;
            }
            if (!intersected) {
                break;
            }
            po.setX(((int)(Math.floor(po.getX()*16) + i))/16.0d);
            po.setY(((int)(Math.floor(po.getY()*16) + i0))/16.0d);
            time++;
        }
        return time;
    }

    public boolean[] doUpdate(IEventCallback ev, IPhysicsObject ipo,
            GameState gs, double InGameTime) {
        boolean[] aa = new boolean[4];
        ipo.setX(ipo.getX() + ipo.getVX());
        boolean[] b = collisionPass(ipo, gs, InGameTime, ev, false);
        cancelVelocityInDirections(b, ipo);
        aa[0] = aa[0] || b[0];
        aa[1] = aa[1] || b[1];
        aa[2] = aa[2] || b[2];
        aa[3] = aa[3] || b[3];
        ipo.setY(ipo.getY() + ipo.getVY());
        b = collisionPass(ipo, gs, InGameTime, ev, false);
        cancelVelocityInDirections(b, ipo);
        aa[0] = aa[0] || b[0];
        aa[1] = aa[1] || b[1];
        aa[2] = aa[2] || b[2];
        aa[3] = aa[3] || b[3];
        b = collisionPass(ipo, gs, InGameTime, ev, true);
        cancelVelocityInDirections(b, ipo);
        aa[0] = aa[0] || b[0];
        aa[1] = aa[1] || b[1];
        aa[2] = aa[2] || b[2];
        aa[3] = aa[3] || b[3];
        return aa;
    }

    // 0:Left,1:Right,2:Up,3:Down
    // Note that these directions are the directions that the player was pushed
    // in to move them out of a block.
    private void cancelVelocityInDirections(boolean[] dirs, IPhysicsObject op) {
        if (dirs[0] && (op.getVX()>0))
            op.setVX(0);
        if (dirs[1] && (op.getVX()<0))
            op.setVX(0);
        if (dirs[2] && (op.getVY()>0))
            op.setVY(0);
        if (dirs[3] && (op.getVY()<0))
            op.setVY(0);
    }
}
