/*
 * Part of the JavaChan project.
 * (Java ikaChan)
 *
 *        DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *                    Version 2, December 2004 
 *
 * Modified by gamemanj <asdd2808@gmail.com> to remove expletives
 * Original copyright (C) 2004 Sam Hocevar <sam@hocevar.net> 
 *
 * Everyone is permitted to copy and distribute verbatim or modified 
 * copies of this license document, and changing it is allowed as long 
 * as the name is changed. 
 *
 *            DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION 
 *
 *  0. You just DO WHATEVER YOU WANT TO.
 */
package javachan.games;

import java.io.FileOutputStream;
import javachan.Game;
import javachan.GameState;
import javachan.GameStateNPChar.NPCCharacter;
import javachan.IGame;
import gabien.IGrInDriver;
import gabien.IGrInDriver.IImage;
import gabien.ui.Rect;
import gabien.ui.UIButton;
import gabien.ui.UIPanel;
import javachan.blocks.BasicBlockType;
import javachan.blocks.DirectionBlockType;
import javachan.games.editorui.UIHorizontalList;
import javachan.games.editorui.UIHorizontalList.ListEntry;
import javachan.games.editorui.UINumberBox;
import javachan.npcs.BasicNPCType;

/**
 * 
 * Yep,there's a LOT of code here. Note that the layout code is still pretty
 * horrible. However,it's a lot less horrible than it was before. ('before'
 * means a time where everything was done by insanely long expressions.
 * Occasionally I'd use a javachan.Rect to keep stuff in shape. See src folder
 * of v1 to v6.1,or if you don't have it,ask for it via PM. v6 is missing due to
 * the fact that v6.1 was correcting a packaging error.)
 * 
 * @author matt
 */
// NEW(1x GUI,as recommended by
// andwhyisit.)_____________________________________
// Note the NPC editor layout is NOW like this:
/*
 * NPCW NPCTW MPW 128 | width-(MPW+NPCW) | 96 ----------------------------------
 * so NPCTW now makes the sacrifice. and vertically: MPH 64
 */
// OLD_____________________________________
// Note the NPC editor layout is like this:
/*
 * NPCW NPCTW MPW 64 | 216-NPCW | width-(NPCW+NPCTW)
 * ----------------------------------
 * 
 * and vertically: MPH 32
 */

public class EditorGame implements IGame {

    private static class TileListEntry implements ListEntry {
        public int tleType, tleSubtype;
        public boolean NoSubtypes;
        public BasicBlockType blockType;
        public double internalClock = 0;// kludge:we don't receive TSS time,we
                                        // only get delta

        public TileListEntry(int A, int B, boolean C, BasicBlockType[] BT) {
            blockType = BT[A];
            tleType = A;
            tleSubtype = B;
            NoSubtypes = C;
        }

        @Override
        public boolean isEqualTo(ListEntry le) {
            if (((TileListEntry) le).tleType == tleType)
                if (NoSubtypes || ((TileListEntry) le).NoSubtypes
                        || (((TileListEntry) le).tleSubtype == tleSubtype))
                    return true;
            return false;
        }

        @Override
        public void doRender(int centerx, int centery, double DeltaTime,
                IGrInDriver igd) {

            internalClock += DeltaTime;
            if (blockType != null) {
                igd.blitImage(160 + (tleType * 8), 48, 16, 16, centerx - 8,
                        centery - 24, igd.getImage("TileEditor.png", 0, 0, 0));
                if (blockType instanceof DirectionBlockType) {
                    igd.blitImage(16 + (16 * tleSubtype), 32, 16, 16, centerx - 8,
                            centery - 8,
                            igd.getImage("TileEditor.png", 0, 0, 0));
                }
                blockType.doRender(0, centerx - 8, centery - 8, tleSubtype, igd,
                        internalClock);
                blockType.doRender(1, centerx - 8, centery - 8, tleSubtype, igd,
                        internalClock);
            } else {
                igd.blitImage(0, 32, 16, 16, centerx - 8, centery - 8,
                        igd.getImage("TileEditor.png", 0, 0, 0));
            }
        }
    }

    private static class NPCListEntry implements ListEntry {
        public int npcType, npcSubtype;
        public boolean NoSubtypes;
        public GameState gs;

        public NPCListEntry(int A, int B, boolean C, GameState gs) {
            npcType = A;
            npcSubtype = B;
            NoSubtypes = C;
            this.gs = gs;
        }

        @Override
        public boolean isEqualTo(ListEntry le) {
            if (((NPCListEntry) le).npcType == npcType)
                if (NoSubtypes || ((NPCListEntry) le).NoSubtypes
                        || (((NPCListEntry) le).npcSubtype == npcSubtype))
                    return true;
            return false;
        }

        @Override
        public void doRender(int centerx, int centery, double DeltaTime,
                IGrInDriver igd) {
            NPCCharacter npcc = new NPCCharacter();
            npcc.exists = true;
            npcc.entityType = npcType;
            npcc.ofsX = npcSubtype;
            BasicNPCType bnt = gs.npcs.npcTypes[npcc.entityType];
            bnt.Draw(centerx - 8, centery - 8, DeltaTime, npcc, gs, igd);
        }
    }

    // This is for the really nice NPC scroll thing.
    // For IdioticBaka1824
    public final int[] npcTypes = { 0, 1, 1, 1, 1, 1, 2, 3, 4, 5, 6, 6, 6, 7,
            8, 9 };
    public final int[] npcSubTypes = { 0, 0, 1, 2, 3, 4, 0, 0, 0, 0, 0, 1, 2,
            0, 0, 0 };
    public final boolean[] npcSubTypeLess = { true, false, true, true, true,
            true, false, true, true, true };
    // Part of the BCM update(BCM:Big Code Merge-unifying horizontal list
    // code,finally,since 2014).
    public final int[] tilesTypes = { 0, 2, 2, 2, 2, 4, 6, 6, 6, 6, 8, 8, 8, 8,
            8, 8, 8, 8, 10, 10, 14 };
    public final int[] tilesSubTypes = { 0, 0, 1, 2, 3, 0, 0, 1, 2, 3, 0, 1, 2,
            3, 4, 5, 6, 7, 0, 1, 0 };
    public final boolean[] tilesSubTypeLess = { true, false, false, false,
            false, false, false, false, false, false, false, false, false,
            false, false, false };

    public GameState gs;
    public int currentTab = 0;
    // LEVEL 1 ELEMENTS
    public UIPanel tabNPC = new UIPanel(), tabTile = new UIPanel(),
            tabCommon = new UIPanel();
    // LEVEL 2 ELEMENTS
    // common
    public UIButton buttonSave = new UIButton(), buttonSwitch = new UIButton();
    // npc
    public UIPanel panelNumbers = new UIPanel();
    public UIHorizontalList hlNPCS = new UIHorizontalList();
    // tiles
    public UINumberBox tileNumber = new UINumberBox();
    public UIHorizontalList hlTILES = new UIHorizontalList();
    // LEVEL 3 ELEMENTS
    // npc
    public UIPanel npcCheckBox = new UIPanel();
    public UIButton npcCollisionType = new UIButton();
    public UINumberBox npcEventID = new UINumberBox(),
            npcIdDisplay = new UINumberBox(), npcX = new UINumberBox(),
            npcY = new UINumberBox();
    // PX,PY:Camera position(as draw offset)
    public double camX = 0, camY = 0;
    // If 0 or under,then draw save normally,else draw it offset a bit so the
    // user knows it saved
    public double savePressTime = 0;
    public int selectedNPC = -1;

    public void selectNPC(int i) {
        if (i >= 100)
            i = -1;
        if (i < -1)
            i = -1;
        selectedNPC = i;
        npcIdDisplay.number = i;
        if (i == -1) {
            npcEventID.readOnly = true;
            return;
        }
        npcEventID.readOnly = false;
        npcEventID.number = gs.npcs.npcTable[selectedNPC].eventID;
        NPCListEntry cmp = new NPCListEntry(gs.npcs.npcTable[i].entityType,
                (int) gs.npcs.npcTable[i].ofsX, false, gs);
        for (int p = 0; p < hlNPCS.availableTypes.length; p++) {
            if (cmp.isEqualTo(hlNPCS.availableTypes[p]))
                hlNPCS.selectedItem = p;
        }
        panelNumbers.selectedElement = null;
    }

    public EditorGame(GameState ggs) {

        gs = ggs;
        hlNPCS.availableTypes = new UIHorizontalList.ListEntry[npcTypes.length];
        for (int x = 0; x < npcTypes.length; x++) {
            hlNPCS.availableTypes[x] = new NPCListEntry(npcTypes[x], npcSubTypes[x],
                    npcSubTypeLess[npcTypes[x]], gs);
        }
        hlTILES.availableTypes = new UIHorizontalList.ListEntry[tilesTypes.length];
        for (int x = 0; x < tilesTypes.length; x++) {
            hlTILES.availableTypes[x] = new TileListEntry(tilesTypes[x], tilesSubTypes[x],
                    tilesSubTypeLess[tilesTypes[x]], gs.map.blockTypes);
        }
        panelNumbers.setBounds(new Rect(0, 0, 128, 64));
        hlTILES.onChange = new Runnable() {

            @Override
            public void run() {
                tileNumber.number = (((TileListEntry) hlTILES.availableTypes[hlTILES.selectedItem]).tleType * 16)
                        + (((TileListEntry) hlTILES.availableTypes[hlTILES.selectedItem]).tleSubtype);
            }
        };
        npcCheckBox.setBounds(new Rect(2, 10, 16, 8));
        npcCheckBox.imageX = 304;
        npcCollisionType.setBounds(new Rect(40, 34, 48, 14));
        npcCollisionType.onClick = new Runnable() {

            @Override
            public void run() {
                if (selectedNPC == -1)
                    return;
                gs.npcs.npcTable[selectedNPC].collisionType++;
                if (gs.npcs.npcTable[selectedNPC].collisionType >= 4)
                    gs.npcs.npcTable[selectedNPC].collisionType = 0;
            }
        };
        npcIdDisplay.setBounds(new Rect(40, 0, 64, 16));
        npcIdDisplay.readOnly = true;
        hlNPCS.onChange = new Runnable() {

            @Override
            public void run() {
                if (selectedNPC == -1)
                    return;
                gs.npcs.npcTable[selectedNPC].entityType = ((NPCListEntry) hlNPCS.availableTypes[hlNPCS.selectedItem]).npcType;
                gs.npcs.npcTable[selectedNPC].ofsX = ((NPCListEntry) hlNPCS.availableTypes[hlNPCS.selectedItem]).npcSubtype;
            }
        };
        npcEventID.setBounds(new Rect(40, 16, 64, 16));
        npcEventID.onEdit = new Runnable() {

            @Override
            public void run() {
                if (npcEventID.number >= 256)
                    npcEventID.number = 0;
                if (npcEventID.number < 0)
                    npcEventID.number = 0;
                if (selectedNPC == -1)
                    return;
                gs.npcs.npcTable[selectedNPC].eventID = npcEventID.number;
            }
        };
        npcX.setBounds(new Rect(28, 50, 64, 16));
        npcY.setBounds(new Rect(94, 50, 64, 16));
        npcX.readOnly = true;
        npcY.readOnly = true;
        selectNPC(-1); // initialize the disable/enable stuff

        tabNPC.allElements.add(tabCommon);
        tabNPC.allElements.add(panelNumbers);
        tabNPC.allElements.add(hlNPCS);
        panelNumbers.allElements.add(npcEventID);
        panelNumbers.allElements.add(npcIdDisplay);
        panelNumbers.allElements.add(npcX);
        panelNumbers.allElements.add(npcY);
        panelNumbers.allElements.add(npcCheckBox);
        panelNumbers.allElements.add(npcCollisionType);

        tabTile.allElements.add(tabCommon);
        hlTILES.allElements.add(tileNumber);
        tabTile.allElements.add(hlTILES);
        buttonSave.setBounds(new Rect(0, 0, 32, 32));
        buttonSave.onClick = new Runnable() {

            @Override
            public void run() {
                try {
                    FileOutputStream fos = new FileOutputStream("Pbm/Map1.pbm");
                    gs.map.saveBitmap(fos);
                    fos.close();
                    fos = new FileOutputStream("NPChar.dat");
                    gs.npcs.save(fos);
                    fos.close();
                } catch (Exception e) {
                }
            }
        };
        buttonSwitch.onClick = new Runnable() {

            @Override
            public void run() {
                currentTab ^= 1;
            }
        };

        buttonSwitch.setBounds(new Rect(32, 0, 64, 32));
        tabCommon.allElements.add(buttonSave);
        tabCommon.allElements.add(buttonSwitch);
    }

    @Override
    public void doUpdate(double deltaTime, double mainTime, Game g) {
        // ---------------------------------START MY LAYOUT CODE
        tabCommon.setBounds(new Rect(g.graphics.getWidth() - 96, 0, 96, 64));
        tabNPC.setBounds(new Rect(0, 0, g.graphics.getWidth(), 64));
        tabTile.setBounds(new Rect(0, 0, g.graphics.getWidth(), 64));
        tileNumber.setBounds(new Rect(tabTile.getBounds().width - 64, tabTile
                .getBounds().height - 16, 64, 16));
        // 128(NumbersPanel Size)+96(Common Size)
        hlNPCS.setBounds(new Rect(128, 0, g.graphics.getWidth() - (128 + 96), 64));
        // 96(Common Size)
        hlTILES.setBounds(new Rect(0, 0, g.graphics.getWidth() - 96, 64));
        IImage edit = g.graphics.getImage("TileEditor.png", 0, 0, 0);
        npcCollisionType.buttonImage = npcCheckBox.baseImage = panelNumbers.baseImage = buttonSwitch.buttonImage = buttonSave.buttonImage = edit;
        npcCollisionType.imageX = 448;
        npcCheckBox.imageY = 8;
        //Ensure that the Numbers Panel isn't visible when it shouldn't be.
        //(Note that the NPC selector itself remains visible for selecting a type.
        tabNPC.allElements.remove(panelNumbers);
        tabNPC.allElements.remove(hlNPCS);
        if (selectedNPC != -1)
        {
            if (gs.npcs.npcTable[selectedNPC].exists)
            {
                npcCollisionType.imageY = (gs.npcs.npcTable[selectedNPC].collisionType * 16) + 2;
                npcCheckBox.imageY = 0;
                npcX.number = (int) gs.npcs.npcTable[selectedNPC].posX;
                npcY.number = (int) gs.npcs.npcTable[selectedNPC].posY;
                tabNPC.allElements.add(panelNumbers);
                tabNPC.allElements.add(hlNPCS);
            }
        }
        panelNumbers.imageX = 320;
        buttonSwitch.imageX = 32 + (64 * currentTab);
        // --------------------------------END MY LAYOUT CODE
        g.graphics.clearAll(0, 0, 0);
        int cmx = g.graphics.getMouseX();
        int cmy = g.graphics.getMouseY();
        int cmb = g.graphics.getMouseButton();
        int Mx = (int) Math.floor((cmx - camX) / 16);// in !tiles!
        int My = (int) Math.floor((cmy - camY) / 16);// in !tiles!
        if (g.graphics.getMouseJustDown()) {
            if (cmy < 64) {
                g.graphics.setTypeBuffer("");
                if (currentTab == 0) {
                    tabNPC.handleClick(cmx, cmy, g.graphics.getMouseButton());
                } else {
                    tabTile.handleClick(cmx, cmy, g.graphics.getMouseButton());
                }
            } else {
                if (currentTab == 0) {
                    if (cmb == 1) {
                        // CE behavior: Left click is select. No matter what,
                        // the first thing it does is select.

                        int op = selectedNPC;
                        selectNPC(-1);
                        int p = (op + 1) % gs.npcs.npcTable.length;
                        while (p!=op) {
                            NPCCharacter npcc = gs.npcs.npcTable[p];
                            if (npcc.exists)
                                if (new Rect(Mx * 16, My * 16, 16, 16)
                                        .intersects(new Rect(
                                                (int) (npcc.posX * 16),
                                                (int) (npcc.posY * 16), 16, 16))) {
                                    selectNPC(p);
                                    break;
                                }
                            p++;
                            if (p >= gs.npcs.npcTable.length)
                            {
                                p = 0;
                                if (op==-1)
                                    break;
                            }
                        }
                        //p cannot == op if op is -1
                        if (p==op)
                        {
                            NPCCharacter npcc=gs.npcs.npcTable[op];
                            //*should* be the case, but still
                            if (npcc.exists)
                            {
                                if (new Rect(Mx * 16, My * 16, 16, 16)
                                .intersects(new Rect(
                                        (int) (npcc.posX * 16),
                                        (int) (npcc.posY * 16), 16, 16))) {
                                selectNPC(op);
                                }
                            } else {
                                System.out.println("Npc was selected that didn't exist?");
                            }
                        }
                    }
                    if (cmb == 3) {
                        g.gameState=new EditorNpcMenuGame(this,Mx,My);
                        g.graphics.clearKeys();
                    }
                } else {
                    if (cmb == 1) {
                        if (Mx >= 0)
                            if (My >= 0)
                                if (Mx < gs.map.width)
                                    if (My < gs.map.height)
                                        gs.map.data[Mx + (My * gs.map.width)] = tileNumber.number;
                    } else {
                        if (Mx >= 0)
                            if (My >= 0)
                                if (Mx < gs.map.width)
                                    if (My < gs.map.height)
                                        gs.map.data[Mx + (My * gs.map.width)] = 0;
                    }
                }
            }
        }
        if (g.graphics.getMouseDown()) {
            if (cmy < 64) {
                if (cmb == 1) {
                    if (currentTab == 0) {
                        tabNPC.handleDrag(cmx, cmy);
                    } else {
                        tabTile.handleDrag(cmx, cmy);
                    }
                }
            } else {
                if (currentTab == 0) {
                    //Oh, good, I left a area here for later expansion. This is simple enough...
                    if (selectedNPC!=-1)
                    {
                        gs.npcs.npcTable[selectedNPC].posX=Mx;
                        gs.npcs.npcTable[selectedNPC].posY=My;
                    }
                } else {
                }
            }
        }
        if (g.graphics.isKeyJustPressed(Keymap.VK_SWIM)) {
            if (currentTab == 0) {

            }
        }
        if (g.graphics.isKeyDown(IGrInDriver.VK_DOWN)) {
            camY -= deltaTime * 256;
        }
        if (g.graphics.isKeyDown(IGrInDriver.VK_UP)) {
            camY += deltaTime * 256;
        }
        if (g.graphics.isKeyDown(IGrInDriver.VK_LEFT)) {
            camX += deltaTime * 256;
        }
        if (g.graphics.isKeyDown(IGrInDriver.VK_RIGHT)) {
            camX -= deltaTime * 256;
        }
        gs.renderBack((int) camX, (int) camY, mainTime, g.graphics);
        gs.map.renderTiles((int) camX, (int) camY, 0, mainTime, g.graphics);
        for (int xx = -1; xx < gs.map.width + 1; xx++) {
            for (int yy = -1; yy < gs.map.height + 1; yy++) {
                if (new Rect(0, 0, gs.map.width, gs.map.height)
                        .contains(xx, yy))
                    continue;
                g.graphics.blitImage(288, 0, 16, 16, ((xx * 16) + ((int) camX)),
                        ((yy * 16) + ((int) camY)), edit);
            }
        }
        gs.npcs.render((int) camX, (int) camY, mainTime, gs, g.graphics);
        gs.renderPC((int) camX, (int) camY, 0, g.graphics, mainTime);
        gs.map.renderTiles((int) camX, (int) camY, 1, mainTime, g.graphics);
        g.graphics.blitBCKImage(0, 32, 16, 16, ((Mx * 16) + ((int) camX)),
                ((My * 16) + ((int) camY)), edit);
        if (currentTab == 0) {
            if (selectedNPC != -1)
                g.graphics.blitBCKImage(32, 16, 16, 16,
                        (int) (gs.npcs.npcTable[selectedNPC].posX * 16) + (int) camX,
                        (int) (gs.npcs.npcTable[selectedNPC].posY * 16) + (int) camY,
                        g.graphics.getImage("Pbm/Cursor.pbm", 0, 0, 0));
            for (int pp = 0; pp < gs.npcs.npcTable.length; pp++) {
                if (gs.npcs.npcTable[pp].exists)
                    g.graphics.blitBCKImage(32, 0, 16, 16,
                            (int) (gs.npcs.npcTable[pp].posX * 16) + (int) camX,
                            (int) (gs.npcs.npcTable[pp].posY * 16) + (int) camY,
                            g.graphics.getImage("Pbm/Cursor.pbm", 0, 0, 0));
            }
        }
        displayNumber(Mx, 0, g.graphics.getHeight() - 16, g.graphics, edit);
        displayNumber(My, 64, g.graphics.getHeight() - 16, g.graphics, edit);
        if (currentTab == 0) {
            tabTile.selectedElement = null;
            tabNPC.updateAndRender(0, 0, deltaTime, true, g.graphics);
        } else {
            tabNPC.selectedElement = null;
            tabTile.updateAndRender(0, 0, deltaTime, true, g.graphics);
        }
        g.graphics.flush();
    }
    //Menu option calls
    public void createNPC(int mx, int my) {
        for (int i=0;i<gs.npcs.npcTable.length;i++)
        {
            if (!gs.npcs.npcTable[i].exists)
            {
                gs.npcs.npcTable[i].exists=true;
                gs.npcs.npcTable[i].entityType = ((NPCListEntry) hlNPCS.availableTypes[hlNPCS.selectedItem]).npcType;
                gs.npcs.npcTable[i].ofsX = ((NPCListEntry) hlNPCS.availableTypes[hlNPCS.selectedItem]).npcSubtype;
                gs.npcs.npcTable[i].ofsY=0;
                gs.npcs.npcTable[i].posX=mx;
                gs.npcs.npcTable[i].posY=my;
                gs.npcs.npcTable[i].entityStatus=0;
                gs.npcs.npcTable[i].entityStatus2=0;
                gs.npcs.npcTable[i].collisionType=0;
                selectNPC(i);
                return;
            }
        }
    }
    public void deleteNPC()
    {
        gs.npcs.npcTable[selectedNPC].exists = false;
        selectedNPC=-1;
    }
    
    public static void displayNumber(int NPC_NO, int i, int i0,
            IGrInDriver graphics, IImage edit) {
        String ss = "" + NPC_NO;
        int pp = 0;
        for (char c : ss.toCharArray()) {
            int ind=c-'0';
            if (c=='-')
                ind=10;
            graphics.blitImage(ind * 8, 48, 8, 16, i + pp, i0, edit);
            pp += 9;
        }
    }
}
