/*
 * Part of the JavaChan project.
 * (Java ikaChan)
 *
 *        DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *                    Version 2, December 2004 
 *
 * Modified by gamemanj <asdd2808@gmail.com> to remove expletives
 * Original copyright (C) 2004 Sam Hocevar <sam@hocevar.net> 
 *
 * Everyone is permitted to copy and distribute verbatim or modified 
 * copies of this license document, and changing it is allowed as long 
 * as the name is changed. 
 *
 *            DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION 
 *
 *  0. You just DO WHATEVER YOU WANT TO.
 */
package javachan.music;

import java.io.DataInputStream;
import java.io.IOException;
import java.io.InputStream;

import gabien.GaBIEn;
import gabien.ISoundDriver;

public class PiyoPiyoPlayingSample {

    // Sample indexes
    public static int[] drumTable = new int[] { 0, 1, 4, -1, 2, 3, 5, -1, -1,
            -1, -1, -1 };

    public static short[][] samples = new short[6][];

    public ISoundDriver.IChannel channel;

    public static void ensureAllSamplesLoaded() throws IOException {
        InputStream is = GaBIEn.getFile("piyoDrums.bin");
        DataInputStream dis = new DataInputStream(is);
        for (int drumSample = 0; drumSample < 6; drumSample++) {
            short[] sample = new short[dis.readInt() / 2];
            for (int i = 0; i < sample.length; i++) {
                int l = dis.read();
                sample[i] = (short) (dis.read() << 8);
                sample[i] |= l;
            }
            samples[drumSample] = sample;
        }
        is.close();
    }

    public static short[] getSample(int pos) {
        int drumSample = drumTable[pos / 2];
        if (drumSample == -1)
            return new short[1];
        if (samples[drumSample] == null) {
            throw new RuntimeException("Load the samples before using drums!");
        }
        return samples[drumSample];
    }

    public PiyoPiyoPlayingSample(ISoundDriver.IChannel channel, int pos,
            int volume, int pan) {
        boolean quiet = (pos % 2) == 0;
        double lValue = PiyoPiyoPlayingNote.doPan(volume, pan, true);
        double rValue = PiyoPiyoPlayingNote.doPan(volume, pan, false);
        if (quiet) {
            lValue *= 0.75;
            rValue *= 0.75;
        } else {
            lValue *= 0.50;
            rValue *= 0.50;
        }
        this.channel = channel;
        short[] sample = getSample(pos);
        channel.playSound(1, lValue, rValue, sample, false);
    }

    public boolean doUpdate(double deltaTime) {
        if (channel.volL() == 0) {
            return true;
        }
        return false;
    }
}
